@@ -21,24 +21,22 @@
 
 from __future__ import print_function
 
-import sys
-import os
 import errno
+import os
+import signal
 import stat
 import subprocess
-import time
+import sys
 import syslog
-import signal
+import time
 
-import debconf
 import apt_pkg
 from apt.cache import Cache
+import debconf
 
 sys.path.insert(0, '/usr/lib/ubiquity')
 
-from ubiquity import misc
-from ubiquity import install_misc
-from ubiquity import osextras
+from ubiquity import install_misc, misc, osextras
 
 
 class Install(install_misc.InstallBase):
@@ -270,24 +268,16 @@ class Install(install_misc.InstallBase):
             if subarch == 'efi':
                 keep.add('grub-efi')
                 keep.add('grub-efi-amd64')
-                efi_vars = "/sys/firmware/efi/vars"
-                sb_var = os.path.join(
-                    efi_vars,
-                    "SecureBoot-8be4df61-93ca-11d2-aa0d-00e098032b8c",
-                    "data")
-                if os.path.exists(sb_var):
-                    with open(sb_var, "rb") as sb_var_file:
-                        if sb_var_file.read(1) == b"\x01":
-                            keep.add('grub-efi-amd64-signed')
-                            keep.add('shim-signed')
-                            try:
-                                altmeta = self.db.get(
-                                    'base-installer/kernel/altmeta')
-                                if altmeta:
-                                    altmeta = '-%s' % altmeta
-                            except debconf.DebconfError:
-                                altmeta = ''
-                            keep.add('linux-signed-generic%s' % altmeta)
+                keep.add('grub-efi-amd64-signed')
+                keep.add('shim-signed')
+                try:
+                    altmeta = self.db.get(
+                        'base-installer/kernel/altmeta')
+                    if altmeta:
+                        altmeta = '-%s' % altmeta
+                except debconf.DebconfError:
+                    altmeta = ''
+                keep.add('linux-signed-generic%s' % altmeta)
             else:
                 keep.add('grub')
                 keep.add('grub-pc')
@@ -318,19 +308,21 @@ class Install(install_misc.InstallBase):
             if not os.path.exists('/var/lib/dpkg/info/%s.prerm' % x)}
 
         confirmed_remove = set()
-        for pkg in sorted(difference):
-            if pkg in confirmed_remove:
-                continue
-            would_remove = install_misc.get_remove_list(
-                cache, [pkg], recursive=True)
-            if would_remove <= difference:
-                confirmed_remove |= would_remove
-                # Leave these marked for removal in the apt cache to speed
-                # up further calculations.
-            else:
-                for removedpkg in would_remove:
-                    cachedpkg = install_misc.get_cache_pkg(cache, removedpkg)
-                    cachedpkg.mark_keep()
+        with cache.actiongroup():
+            for pkg in sorted(difference):
+                if pkg in confirmed_remove:
+                    continue
+                would_remove = install_misc.get_remove_list(
+                    cache, [pkg], recursive=True)
+                if would_remove <= difference:
+                    confirmed_remove |= would_remove
+                    # Leave these marked for removal in the apt cache to
+                    # speed up further calculations.
+                else:
+                    for removedpkg in would_remove:
+                        cachedpkg = install_misc.get_cache_pkg(
+                            cache, removedpkg)
+                        cachedpkg.mark_keep()
         difference = confirmed_remove
 
         if len(difference) == 0:
