@@ -21,33 +21,30 @@
 
 from __future__ import print_function
 
-import sys
+import fcntl
+import gzip
 import os
 import platform
-import stat
-import re
-import textwrap
-import shutil
-import subprocess
-import struct
-import socket
-import fcntl
-import traceback
-import syslog
-import gzip
 import pwd
+import re
+import shutil
+import socket
+import stat
+import struct
+import subprocess
+import sys
+import syslog
+import textwrap
+import traceback
 
-import debconf
 import apt_pkg
 from apt.cache import Cache
+import debconf
 
 sys.path.insert(0, '/usr/lib/ubiquity')
 
-from ubiquity import misc
-from ubiquity import install_misc
-from ubiquity import osextras
-from ubiquity import plugin_manager
-from ubiquity.components import apt_setup, hw_detect, check_kernels
+from ubiquity import install_misc, misc, osextras, plugin_manager
+from ubiquity.components import apt_setup, check_kernels, hw_detect
 
 
 INTERFACES_TEXT = """\
@@ -313,9 +310,8 @@ class Install(install_misc.InstallBase):
 
         self.db.progress('SET', self.end)
 
-    def configure_face(self):
-        PHOTO_PATH = '/var/lib/ubiquity/webcam_photo.png'
-        target_user = self.db.get('passwd/username')
+    def _get_uid_gid_on_target(self, target_user):
+        """Helper that gets the uid/gid of the username in the target chroot"""
         uid = subprocess.Popen(
             ['chroot', self.target, 'sudo', '-u', target_user, '--',
              'id', '-u'], stdout=subprocess.PIPE, universal_newlines=True)
@@ -328,8 +324,14 @@ class Install(install_misc.InstallBase):
             uid = int(uid)
             gid = int(gid)
         except ValueError:
-            return
-        if os.path.exists(PHOTO_PATH):
+            return (None, None)
+        return uid, gid
+
+    def configure_face(self):
+        PHOTO_PATH = '/var/lib/ubiquity/webcam_photo.png'
+        target_user = self.db.get('passwd/username')
+        uid, gid = self._get_uid_gid_on_target(target_user)
+        if os.path.exists(PHOTO_PATH) and uid and gid:
             targetpath = self.target_file('home', target_user, '.face')
             shutil.copy2(PHOTO_PATH, targetpath)
             os.lchown(targetpath, uid, gid)
@@ -1094,7 +1096,8 @@ class Install(install_misc.InstallBase):
             self.nested_progress_end()
             return
 
-        install_misc.get_remove_list(cache, to_remove, recursive)
+        with cache.actiongroup():
+            install_misc.get_remove_list(cache, to_remove, recursive)
 
         self.db.progress('SET', 1)
         self.progress_region(1, 5)
