@@ -87,9 +87,7 @@ class PartMan(QtGui.QWidget):
                 # data for bar display
                 size = int(partition['parted']['size'])
                 fs = partition['parted']['fs']
-                path = partition['parted']['path'].replace("/dev/", "")
-                if fs == "free":
-                    path = fs
+                path = partition['parted']['path']
                 partition_bar.addPartition(path, size, fs)
 
         self.partition_list_treeview.reset()
@@ -180,6 +178,10 @@ class PartMan(QtGui.QWidget):
         dialog.partition_create_place_beginning.setChecked(True)
 
         self.create_use_method_names = {}
+
+        # Remove any previous entries
+        dialog.partition_create_use_combo.clear()
+
         for method, name, description in (
                 self.ctrlr.dbfilter.use_as(devpart, True)):
             self.create_use_method_names[description] = name
@@ -227,7 +229,7 @@ class PartMan(QtGui.QWidget):
         if not hasattr(self, 'create_use_method_names'):
             return
         known_filesystems = ('ext4', 'ext3', 'ext2',
-                             'btrfs', 'reiserfs', 'jfs', 'xfs',
+                             'btrfs', 'jfs', 'xfs',
                              'fat16', 'fat32', 'ntfs', 'uboot')
         text = str(self.create_dialog.partition_create_use_combo.currentText())
         if text not in self.create_use_method_names:
@@ -371,7 +373,7 @@ class PartMan(QtGui.QWidget):
         # point makes no sense. TODO cjwatson 2007-01-31: Unfortunately we
         # have to hardcode the list of known filesystems here.
         known_filesystems = ('ext4', 'ext3', 'ext2',
-                             'btrfs', 'reiserfs', 'jfs', 'xfs',
+                             'btrfs', 'jfs', 'xfs',
                              'fat16', 'fat32', 'ntfs', 'uboot')
         text = str(self.edit_dialog.partition_edit_use_combo.currentText())
         if text not in self.edit_use_method_names:
