@@ -41,8 +41,7 @@ from apt.progress.text import AcquireProgress
 import apt_pkg
 import debconf
 
-from ubiquity import misc
-from ubiquity import osextras
+from ubiquity import misc, osextras
 from ubiquity.casper import get_casper
 
 
@@ -144,6 +143,7 @@ exit 0""", file=f)
         with open(initctl, 'w') as f:
             print("""\
 #!/bin/sh
+if [ "$1" = version ]; then exec /sbin/initctl.REAL "$@"; fi
 echo 1>&2
 echo 'Warning: Fake initctl called, doing nothing.' 1>&2
 exit 0""", file=f)
@@ -906,8 +906,9 @@ class InstallBase:
             self.nested_progress_end()
             return
 
-        for pkg in to_install:
-            mark_install(cache, pkg)
+        with cache.actiongroup():
+            for pkg in to_install:
+                mark_install(cache, pkg)
 
         self.db.progress('SET', 1)
         self.progress_region(1, 10)
