@@ -50,11 +50,12 @@ def set_groups_for_uid(uid):
 def drop_all_privileges():
     # gconf needs both the UID and effective UID set.
     global _dropped_privileges
-    uid = os.environ.get('SUDO_UID')
-    gid = os.environ.get('SUDO_GID')
+    uid = os.environ.get('PKEXEC_UID')
+    gid = None
     if uid is not None:
         uid = int(uid)
         set_groups_for_uid(uid)
+        gid = pwd.getpwuid(uid).pw_gid
     if gid is not None:
         gid = int(gid)
         os.setregid(gid, gid)
@@ -70,11 +71,12 @@ def drop_privileges():
     global _dropped_privileges
     assert _dropped_privileges is not None
     if _dropped_privileges == 0:
-        uid = os.environ.get('SUDO_UID')
-        gid = os.environ.get('SUDO_GID')
+        uid = os.environ.get('PKEXEC_UID')
+        gid = None
         if uid is not None:
             uid = int(uid)
             set_groups_for_uid(uid)
+            gid = pwd.getpwuid(uid).pw_gid
         if gid is not None:
             gid = int(gid)
             os.setegid(gid)
@@ -98,11 +100,12 @@ def drop_privileges_save():
     # At the moment, we only know how to handle this when effective
     # privileges were already dropped.
     assert _dropped_privileges is not None and _dropped_privileges > 0
-    uid = os.environ.get('SUDO_UID')
-    gid = os.environ.get('SUDO_GID')
+    uid = os.environ.get('PKEXEC_UID')
+    gid = None
     if uid is not None:
         uid = int(uid)
         set_groups_for_uid(uid)
+        gid = pwd.getpwuid(uid).pw_gid
     if gid is not None:
         gid = int(gid)
         os.setresgid(gid, gid, 0)
@@ -294,7 +297,7 @@ def partition_to_disk(partition):
     """Convert a partition device to its disk device, if any."""
     udevadm_part = udevadm_info(['-n', partition])
     if ('DEVPATH' not in udevadm_part or
-        udevadm_part.get('DEVTYPE') != 'partition'):
+            udevadm_part.get('DEVTYPE') != 'partition'):
         return partition
 
     disk_syspath = '/sys%s' % udevadm_part['DEVPATH'].rsplit('/', 1)[0]
@@ -360,7 +363,7 @@ def grub_default(boot=None):
     except OSError:
         same = False
     if ((same or target == '(hd0)') and
-        ((cdfs and cdfs != 'iso9660') or is_removable(cdsrc))):
+            ((cdfs and cdfs != 'iso9660') or is_removable(cdsrc))):
         # Installing from removable media other than a CD.  Make sure that
         # we don't accidentally install GRUB to it.
         boot = boot_device()
@@ -456,7 +459,7 @@ def windows_startup_folder(mount_path):
         'Documents and Settings/All Users/Start Menu/Programs/Startup',
         # Windows NT
         'Winnt/Profiles/All Users/Start Menu/Programs/Startup',
-                ]
+    ]
     for location in locations:
         path = os.path.join(mount_path, location)
         if os.path.exists(path):
@@ -476,6 +479,7 @@ def get_release():
                     line = line.split()
                     if line[2] == 'LTS':
                         line[1] += ' LTS'
+                    line[0] = line[0].replace('-', ' ')
                     get_release.release_info = ReleaseInfo(
                         name=line[0], version=line[1])
         except:
@@ -525,8 +529,8 @@ def get_install_medium():
             else:
                 get_install_medium.medium = 'CD'
         except:
-            syslog.syslog(syslog.LOG_ERR,
-                "Unable to determine install medium.")
+            syslog.syslog(
+                syslog.LOG_ERR, "Unable to determine install medium.")
             get_install_medium.medium = 'CD'
     return get_install_medium.medium
 
@@ -765,17 +769,17 @@ def set_indicator_keymaps(lang):
             "setxkbmap", "-layout", ",".join(kb_layouts),
             "-variant", ",".join(kb_variants))
 
-    iso_639_3 = ElementTree.parse('/usr/share/xml/iso-codes/iso_639_3.xml')
-    nodes = [element for element in iso_639_3.findall('iso_639_3_entry')
-             if element.get('part1_code') == lang]
+    iso_639 = ElementTree.parse('/usr/share/xml/iso-codes/iso_639.xml')
+    nodes = [element for element in iso_639.findall('iso_639_entry')
+             if element.get('iso_639_1_code') == lang]
     display = GdkX11.x11_get_default_xdisplay()
     engine = Xkl.Engine.get_instance(display)
     if nodes:
         configreg = Xkl.ConfigRegistry.get_instance(engine)
         configreg.load(False)
 
-        # Apparently part2_code doesn't always work (fails with French)
-        for prop in ('part2_code', 'id', 'part1_code'):
+        # Apparently iso_639_2B_code doesn't always work (fails with French)
+        for prop in ('iso_639_2B_code', 'iso_639_2T_code', 'iso_639_1_code'):
             code = nodes[0].get(prop)
             if code is not None:
                 configreg.foreach_language_variant(code, process_variant, None)
